@@ -36,6 +36,8 @@ class EffectStackModel;
 class InvalidDialog;
 class TranscodeSeek;
 class KdenliveDoc;
+class KSelectAction;
+class KeyframeModelList;
 class TagWidget;
 class Monitor;
 class ProjectClip;
@@ -190,10 +192,10 @@ class Bin : public QWidget
     Q_OBJECT
 
     /** @brief Defines the view types (icon view, tree view,...)  */
-    enum BinViewType { BinTreeView, BinIconView };
+    enum BinViewType { BinTreeView, BinIconView, BinUnknownView };
 
 public:
-    explicit Bin(std::shared_ptr<ProjectItemModel> model, QWidget *parent = nullptr, bool isMainBin = true);
+    explicit Bin(std::shared_ptr<ProjectItemModel> model, QWidget *parent = nullptr, bool isMainBin = true, BinViewType type = BinUnknownView);
     ~Bin() override;
 
     bool isLoading;
@@ -208,12 +210,6 @@ public:
     /** @brief Used to notify the Model View that an item was updated */
     void emitItemUpdated(std::shared_ptr<AbstractProjectItem> item);
 
-    /** @brief Set monitor associated with this bin (clipmonitor) */
-    void setMonitor(Monitor *monitor);
-
-    /** @brief Returns the clip monitor */
-    Monitor *monitor();
-
     /** @brief Open a producer in the clip monitor */
     void openProducer(std::shared_ptr<ProjectClip> controller);
     void openProducer(std::shared_ptr<ProjectClip> controller, int in, int out);
@@ -268,12 +264,18 @@ public:
     /** @brief Update some timeline related stuff, like used clips bound, audio targets,... */
     void sequenceActivated();
 
-    void doMoveClip(const QString &id, const QString &newParentId);
+    /** @brief Move clips inside the bin (from one folder to another)
+        @param ids a map of ids: {item id, {new parent, old parent}}
+        @param redo if true, change to new parent, otherwise to old parent
+    */
+    void doMoveClips(QMap<QString, std::pair<QString, QString>> ids, bool redo);
+    /** @brief Block/unblock signals from the bin selection model, useful for operations affecting many clips */
+    void blockBin(bool block);
     void doMoveFolder(const QString &id, const QString &newParentId);
     void setupGeneratorMenu();
 
     /** @brief Set focus to the Bin view. */
-    void focusBinView() const;
+    void focusBinView();
     /** @brief Get a string list of all clip ids that are inside a folder defined by id. */
     QStringList getBinFolderClipIds(const QString &id) const;
     /** @brief Build a rename subclip command. */
@@ -329,7 +331,7 @@ public:
      * @param id the updated duration of the timeline clip
      * * @param current the uuid of the currently active timeline
      */
-    void updateSequenceClip(const QUuid &uuid, int duration, int pos);
+    void updateSequenceClip(const QUuid &uuid, int duration, int pos, bool forceUpdate = false);
     /** @brief Update a sequence AV info (has audio/video) */
     void updateSequenceAVType(const QUuid &uuid, int tracksCount);
 
@@ -365,6 +367,23 @@ public:
     const QString buildSequenceClipWithUndo(Fun &undo, Fun &redo, int aTracks = -1, int vTracks = -1, QString suggestedName = QString());
     /** @brief Returns true if the project uses a clip with variable framerate. */
     bool usesVariableFpsClip();
+    void applyClipAssetGroupCommand(int cid, const QString &assetId, const QModelIndex &index, const QString &previousValue, QString value,
+                                    QUndoCommand *command);
+    void applyClipAssetGroupKeyframeCommand(int cid, const QString &assetId, const QModelIndex &index, GenTime pos, const QVariant &previousValue,
+                                            const QVariant &value, int ix, QUndoCommand *command);
+    void applyClipAssetGroupMultiKeyframeCommand(int cid, const QString &assetId, const QList<QModelIndex> &indexes, GenTime pos,
+                                                 const QStringList &sourceValues, const QStringList &values, QUndoCommand *command);
+    int clipAssetGroupInstances(int cid, const QString &assetId);
+    /** @brief Return all similar keyframe models from the selection */
+    QList<std::shared_ptr<KeyframeModelList>> getGroupKeyframeModels(int bid, const QString &assetId);
+    void removeEffectFromGroup(int bid, const QString &assetId, int eid);
+    void disableEffectFromGroup(int cid, const QString &assetId, bool disable, Fun &undo, Fun &redo);
+    /** @brief The root id of the folder that this bin displays */
+    const QString rootFolderId() const;
+    /** @brief Export some basic info (root folder, view type) to a string to save it */
+    const QString binInfoToString() const;
+    /** @brief Load info (root folder, view type) from a string */
+    const QString loadInfo(const QStringList binInfo, const QStringList existingNames);
 
 private Q_SLOTS:
     void slotAddClip();
@@ -428,14 +447,16 @@ private Q_SLOTS:
     void updateTimelineOccurrences();
     /** @brief Set (or unset) the default folder for newly created sequence clips. */
     void setDefaultSequenceFolder(bool enable);
+    /** @brief Set (or unset) the default folder for newly created audio captures. */
+    void setDefaultAudioCaptureFolder(bool enable);
     /** @brief Fetch the filters from the UI and apply them to the proxy model */
     void slotApplyFilters();
+    /** @brief Open a new Bin widget */
+    void slotOpenNewBin();
 
 public Q_SLOTS:
 
     void slotRemoveInvalidClip(const QString &id, bool replace, const QString &errorMessage);
-    /** @brief Reload clip thumbnail - when frame for thumbnail changed */
-    void slotRefreshClipThumbnail(const QString &id);
     void slotDeleteClip();
     void slotItemDoubleClicked(const QModelIndex &ix, const QPoint &pos, uint modifiers);
     void slotSwitchClipProperties(const std::shared_ptr<ProjectClip> &clip);
@@ -514,17 +535,18 @@ private:
     BinListItemDelegate *m_binListViewDelegate;
     std::unique_ptr<ProjectSortProxyModel> m_proxyModel;
     QToolBar *m_toolbar;
-    KdenliveDoc *m_doc;
+    KdenliveDoc *m_doc{nullptr};
     QLineEdit *m_searchLine;
     QToolButton *m_addButton;
-    QMenu *m_extractAudioAction;
-    QAction *m_transcodeAction;
-    QMenu *m_clipsActionsMenu;
-    QAction *m_inTimelineAction;
-    QAction *m_showDate;
-    QAction *m_showDesc;
-    QAction *m_showRating;
-    QAction *m_sortDescend;
+    KSelectAction *m_listTypeAction{nullptr};
+    QMenu *m_extractAudioAction{nullptr};
+    QAction *m_transcodeAction{nullptr};
+    QMenu *m_clipsActionsMenu{nullptr};
+    QAction *m_inTimelineAction{nullptr};
+    QAction *m_showDate{nullptr};
+    QAction *m_showDesc{nullptr};
+    QAction *m_showRating{nullptr};
+    QAction *m_sortDescend{nullptr};
     /** @brief Default view type (icon, tree, ...) */
     BinViewType m_listType;
     /** @brief Default icon size for the views. */
@@ -535,33 +557,33 @@ private:
     QDockWidget *m_propertiesDock;
     QScrollArea *m_propertiesPanel;
     QSlider *m_slider;
-    Monitor *m_monitor;
     QIcon m_blankThumb;
-    QMenu *m_menu;
-    QAction *m_openAction;
-    QAction *m_editAction;
-    QAction *m_reloadAction;
+    QMenu *m_menu{nullptr};
+    QAction *m_openAction{nullptr};
+    QAction *m_editAction{nullptr};
+    QAction *m_reloadAction{nullptr};
     QAction *m_replaceAction{nullptr};
     QAction *m_replaceInTimelineAction{nullptr};
-    QAction *m_duplicateAction;
-    QAction *m_locateAction;
-    QAction *m_proxyAction;
-    QAction *m_deleteAction;
-    QAction *m_openInBin;
-    QAction *m_sequencesFolderAction;
-    QAction *m_addClip;
-    QAction *m_createFolderAction;
-    QAction *m_renameAction;
-    QMenu *m_jobsMenu;
-    QAction *m_cancelJobs;
-    QAction *m_discardCurrentClipJobs;
-    QAction *m_discardPendingJobs;
-    QAction *m_upAction;
-    QAction *m_tagAction;
-    QActionGroup *m_sortGroup;
+    QAction *m_duplicateAction{nullptr};
+    QAction *m_locateAction{nullptr};
+    QAction *m_proxyAction{nullptr};
+    QAction *m_deleteAction{nullptr};
+    QAction *m_openInBin{nullptr};
+    QAction *m_sequencesFolderAction{nullptr};
+    QAction *m_audioCapturesFolderAction{nullptr};
+    QAction *m_addClip{nullptr};
+    QAction *m_createFolderAction{nullptr};
+    QAction *m_renameAction{nullptr};
+    QMenu *m_jobsMenu{nullptr};
+    QAction *m_cancelJobs{nullptr};
+    QAction *m_discardCurrentClipJobs{nullptr};
+    QAction *m_discardPendingJobs{nullptr};
+    QAction *m_upAction{nullptr};
+    QAction *m_tagAction{nullptr};
+    QActionGroup *m_sortGroup{nullptr};
     SmallJobLabel *m_infoLabel;
     TagWidget *m_tagsWidget;
-    QMenu *m_filterMenu;
+    QMenu *m_filterMenu{nullptr};
     QActionGroup m_filterTagGroup;
     QActionGroup m_filterRateGroup;
     QActionGroup m_filterUsageGroup;
@@ -604,6 +626,8 @@ private:
     void showBinInfo();
     /** @brief Find all clip Ids that have a specific tag. */
     const QList<QString> getAllClipsWithTag(const QString &tag);
+    /** @brief Paste effect on a list of clips. */
+    bool doPasteEffect(std::vector<QString> ids, const QStringList &effectData);
 
 Q_SIGNALS:
     void itemUpdated(std::shared_ptr<AbstractProjectItem>);
@@ -611,8 +635,6 @@ Q_SIGNALS:
     /** @brief Save folder info into MLT. */
     void storeFolder(const QString &folderId, const QString &parentId, const QString &oldParentId, const QString &folderName);
     void gotFilterJobResults(const QString &, int, int, stringMap, stringMap);
-    /** @brief Request display of effect stack for a Bin clip. */
-    void requestShowEffectStack(const QString &clipName, std::shared_ptr<EffectStackModel>, QSize frameSize, bool showKeyframes);
     void requestShowClipProperties(const std::shared_ptr<ProjectClip> &clip, bool forceRefresh = false);
     /** @brief Request that the given clip is displayed in the clip monitor */
     void requestClipShow(std::shared_ptr<ProjectClip>);
@@ -627,10 +649,10 @@ Q_SIGNALS:
     /** @brief A clip was updated, request panel update. */
     void refreshPanel(const QString &id);
     /** @brief Upon selection, activate timeline target tracks. */
-    void setupTargets(bool hasVideo, QMap <int, QString> audioStreams);
-    /** @brief A drag event ended, inform timeline. */
-    void processDragEnd();
+    void setupTargets(bool hasVideo, QMap<int, QString> audioStreams);
     void requestBinClose();
     /** @brief Update a timeline tab name on clip rename. */
     void updateTabName(const QUuid &, const QString &);
+    /** @brief Some timeline sequence producers have been updated, refresh their occurrences. */
+    void requestUpdateSequences(QMap<QUuid, int> seqs);
 };
