@@ -404,7 +404,6 @@ void VideoWidget::mousePressEvent(QMouseEvent *event)
         // The event has been handled in qml
         m_swallowDrop = true;
     }
-    // event->accept();
     if ((event->button() & Qt::LeftButton) != 0u) {
         if ((event->modifiers() & Qt::ControlModifier) != 0u) {
             // Pan view
@@ -1082,11 +1081,25 @@ bool VideoWidget::playZone(bool startFromIn, bool loop)
         pCore->displayMessage(i18n("Select a zone to play"), ErrorMessage, 500);
         return false;
     }
+    return playZone(m_proxy->zoneIn(), m_proxy->zoneOut(), startFromIn, loop, true);
+}
+
+bool VideoWidget::loopClip(std::pair<int, int> inOut)
+{
+    if (!m_producer || inOut.second <= inOut.first) {
+        pCore->displayMessage(i18n("Select a clip to play"), ErrorMessage, 500);
+        return false;
+    }
+    return playZone(inOut.first, inOut.second, false, true, false);
+}
+
+bool VideoWidget::playZone(int in, int out, bool startFromIn, bool loop, bool zoneMode)
+{
     double current_speed = m_producer->get_speed();
     m_producer->set_speed(0);
     m_proxy->setSpeed(0);
-    m_loopOut = m_proxy->zoneOut();
-    m_loopIn = m_proxy->zoneIn();
+    m_loopOut = out;
+    m_loopIn = in;
     if (qFuzzyIsNull(current_speed)) {
         if (startFromIn || getCurrentPos() > m_loopOut) {
             m_producer->seek(m_loopIn);
@@ -1096,7 +1109,7 @@ bool VideoWidget::playZone(bool startFromIn, bool loop)
         m_consumer->set("scrub_audio", 0);
         m_consumer->set("refresh", 1);
         m_consumer->set("volume", KdenliveSettings::volume() / 100.);
-    } else if (startFromIn || getCurrentPos() > m_loopOut) {
+    } else if (getCurrentPos() > m_loopOut) {
         // Speed change, purge to reduce latency
         m_consumer->set("refresh", 0);
         m_producer->seek(m_loopIn);
@@ -1104,7 +1117,7 @@ bool VideoWidget::playZone(bool startFromIn, bool loop)
         m_producer->set_speed(1.0);
         m_consumer->set("refresh", 1);
     }
-    m_isZoneMode = true;
+    m_isZoneMode = zoneMode;
     m_isLoopMode = loop;
     return true;
 }
@@ -1125,36 +1138,6 @@ bool VideoWidget::restartConsumer()
     return result != -1;
 }
 
-bool VideoWidget::loopClip(QPoint inOut)
-{
-    if (!m_producer || inOut.y() <= inOut.x()) {
-        pCore->displayMessage(i18n("Select a clip to play"), ErrorMessage, 500);
-        return false;
-    }
-    m_loopIn = inOut.x();
-    double current_speed = m_producer->get_speed();
-    m_producer->set_speed(0);
-    m_proxy->setSpeed(0);
-    m_loopOut = inOut.y();
-    if (qFuzzyIsNull(current_speed)) {
-        m_producer->seek(m_loopIn);
-        m_consumer->start();
-        m_consumer->set("scrub_audio", 0);
-        m_consumer->set("refresh", 1);
-        m_consumer->set("volume", KdenliveSettings::volume() / 100.);
-    } else {
-        // Speed change, purge to reduce latency
-        m_consumer->set("refresh", 0);
-        m_consumer->purge();
-        m_producer->seek(m_loopIn);
-        m_producer->set_speed(1.0);
-        m_consumer->set("refresh", 1);
-    }
-    m_isZoneMode = false;
-    m_isLoopMode = true;
-    return true;
-}
-
 void VideoWidget::resetZoneMode()
 {
     if (!m_isZoneMode && !m_isLoopMode) {
